"""Author Juha Meskanen
Date: 2024-10-26
"""

import yaml
import unittest
from io import StringIO

from masterpiece import MasterPiece, Composite
from masterpiece_yamlformat import YamlFormat


class TestYamlFormat(unittest.TestCase):

    def setUp(self) -> None:
        """Set up a hierarchical structure for testing."""
        self.parent = Composite("parent")
        self.child = MasterPiece("child")
        self.parent.add(self.child)

    def test_serialize(self) -> None:
        """Test serialization of the hierarchical object to yaml."""
        # Create a StringIO object to simulate a file
        stream = StringIO()
        yaml_format = YamlFormat(stream)

        # Serialize the parent object
        yaml_format.serialize(self.parent)

        # Verify the yaml output
        expected_output = yaml.dump(
            self.parent.to_dict(), default_flow_style=False, indent=4
        )
        self.assertEqual(stream.getvalue(), expected_output)

    def test_deserialize(self) -> None:
        """Test deserialization of yaml back to the hierarchical object."""
        # Create a StringIO object to simulate a file
        stream = StringIO()

        # Serialize the parent object to yaml first
        yaml_format = YamlFormat(stream)
        yaml_format.serialize(self.parent)

        # Prepare a new Composite object for deserialization
        new_parent = Composite("")

        # Seek to the beginning of the stream for reading
        stream.seek(0)

        # Deserialize the yaml back to the new parent object
        yaml_format = YamlFormat(stream)
        yaml_format.deserialize(new_parent)

        # Verify that the new_parent object has the same attributes
        self.assertEqual(new_parent.name, self.parent.name)
        self.assertEqual(len(new_parent.children), len(self.parent.children))
        self.assertEqual(new_parent.children[0].name, self.child.name)


if __name__ == "__main__":
    unittest.main()



"""
Yaml serialization format

Author: Juha Meskanen
Date: 2024-10-26
"""

import yaml
from typing import Type
from typing_extensions import override  # for python 3.9 compatibility
from io import IOBase
from masterpiece import MasterPiece, classproperty, Format


class YamlFormat(Format):
    """
    The `YamlFormat` class provides methods for serializing and deserializing objects
    to and from YAML format.

    Features:
    ---------
    - Serializes object attributes to a YAML file or stream.
    - Deserializes object attributes from a YAML file or stream.

    Usage:
    ------
    To use the `YamlFormat`, create an instance by passing the target stream. Then,
    call the `serialize` or `deserialize` method with the appropriate object.

    Example:
    --------
    .. code-block:: python

        from masterpiece.core import YamlFormat, MasterPiece

        # create something to serialize
        piece = SomePiece()

        with open("output.yaml", "w") as f:
            yaml_format = YamlFormat(f)
            yaml_format.serialize(piece)

        with open("output.yaml", "r") as f:
            yaml_format = YamlFormat(f)
            yaml_format.deserialize(piece)
    """

    @override
    def __init__(self, stream: IOBase) -> None:
        """Initialize the YamlFormat with a stream (file object).

        Args:
            stream (Any): The stream to write/read YAML data.
        """
        super().__init__(stream)
        self.indent = 4
        self.default_flow_style = False

    @override
    def serialize(self, obj: MasterPiece) -> None:
        """Serialize the object to the given YAML stream.

        Args:
            obj (Any): The object to serialize.
        """
        yaml.dump(
            obj.to_dict(),
            self.stream,
            default_flow_style=self.default_flow_style,
            indent=self.indent,
        )

    @override
    def deserialize(self, obj: MasterPiece) -> None:
        """Load attributes from the given YAML stream into the object.

        Args:
            obj (Any): The object to deserialize into.
        """
        obj.from_dict(yaml.safe_load(self.stream))

    @override
    def save_configuration(self, clazz: Type[MasterPiece]) -> None:
        """Create class configuration file, if configuration is enabled and
        if the file does not exist yet. See --config startup argument.
        Args:
            clazz (Type[Piece]) class to be saved

        """
        yaml.dump(
            clazz.classattrs_to_dict(),
            self.stream,
            default_flow_style=self.default_flow_style,
            indent=self.indent,
        )

    @override
    def load_configuration(self, clazz: Type[MasterPiece]) -> None:
        """Load class attributes from a Yaml file.
        Args:
            clazz (Type[Piece]) class to be configured
        """
        clazz.classattrs_from_dict(yaml.safe_load(self.stream))

    @classproperty
    def file_extension(self) -> str:
        return ".yaml"



"""Example application for testing 'masterpiece_yaml'
"""

from typing_extensions import override
from masterpiece import Application


class MyApp(Application):
    """Plugin aware application to test the plugin. If the plugin module is properly
    installed, then the 'YamlFormat' should get automatically imported and be available
    for serialization."""

    def __init__(self, name: str = "myhome") -> None:
        """Initialize the home application with the given name.
        Args:
            name (str): The name of the application.
        """
        super().__init__(name)
        self.install_plugins()

    @override
    def run(self) -> None:
        """Start the application."""
        super().run()

        # Print out the instance hierarchy
        self.print()


def main() -> None:
    """Main function, yes, main function!."""

    MyApp.load_plugins()
    home = MyApp("myapp")
    home.run()


if __name__ == "__main__":
    main()
